//Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

//This source code is intended only as a supplement to Microsoft
//Development Tools and/or on-line documentation.  See these other
//materials for detailed information regarding Microsoft code samples.

//THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//PARTICULAR PURPOSE.


namespace SdiTemplate
{
	using System;
	using System.Drawing;
	using System.Drawing.Printing;

	using System.Collections;
	using System.ComponentModel;
	using System.Windows.Forms;
    using System.Collections.Generic;


	/// <summary>
    /// <p>
	/// Main form for Sdi application.
    /// Extend main form for Sdi app from this class.
    /// Modify the extended form in the GUI designer.
	/// </summary>
    public class SdiMainForm : System.Windows.Forms.Form
    {
        protected SdiDoc sdiDoc;
        protected Dictionary<Control, SdiView> hashActive =
            new Dictionary<Control, SdiView>();
        private SdiView m_viewCurrent;

        /// <summary>
        /// Calls constructor of type, using reflection.
        /// <p>
        /// If a view has not been created, then it can be created in the
        /// base class using reflection. This saves the programmer from 
        /// having to construct each view before adding it to a container.
        /// <p>
        /// This method is only used internally.
        /// </summary>
        /// <param name="name">The name to be used to retrieve this view.</param>
        /// <param name="type">The type of the class that for this view.</param>
        /// <returns>Returns the view that is created as SdiView.</returns>
        protected SdiView CreateView(String name, Type type)
        {
            Object[] args = new Object[] { name, this };
            SdiView viewToCreate = (SdiView)Activator.CreateInstance(type, args);
            return viewToCreate;
        }

        
        /// <summary>
        /// Call this method from the main form of an Sdi application to create the 
        /// document for the application.
        /// </summary>
        /// <returns>The document that was created, as SdiDoc.</returns>
        public SdiDoc CreateDocument(Type type)
        {
            Object[] args = new Object[] { };
            Doc = (SdiDoc)Activator.CreateInstance(type, args);
            return Doc;
        }

        /// <summary>
        /// Make a view active in a container.
        /// <p>
        /// This will construct the view if it has not already been constructed.<br>
        /// Call it for each view that will be displayed.<br>
        /// This method will make this view active and hide the previous view.
        /// </summary>
        /// <param name="strViewName">The name used to retrieve the view.</param>
        /// <param name="type">The type of the view class.</param>
        /// <param name="control">The container control that the view should be added to.</param>
        /// <returns>The view that was activated; returned as SdiView.</returns>
        public SdiView SetView(String strViewName, Type type, Control control)
        {
            SdiView viewToSet = GetView(strViewName, type, control);
            if (viewToSet == null)
            {
                viewToSet = CreateView(strViewName, type);
            }
            if (!control.Controls.Contains(viewToSet))
            {
                control.Controls.Add(viewToSet);
            }
            SdiView active = GetActiveView(control);
            if (active != null) active.Hide();
            hashActive[control] = viewToSet;
            m_viewCurrent = viewToSet;
            viewToSet.Update(null);
            viewToSet.BringToFront();
            viewToSet.Show();
            return viewToSet;
        }

        /// <summary>
        /// Retrieve a view by name.
        /// <p>
        /// Uses the name that was sent to the SetView method.
        /// </summary>
        /// <param name="key">The name of the view to retrieve.</param>
        /// <param name="type">The type of the view to retrieve.</param>
        /// <param name="container">The container that contains the view.</param>
        /// <returns>The view; returned as SdiView.</returns>
        public SdiView GetView(String key, Type type, Control container)
        {
            foreach (Control control in container.Controls)
            {
                SdiView view = control as SdiView;
                if (view != null && view.Name.Equals(key))
                {
                    return view;
                }
            }
            return null;
        }

        /// <summary>
        /// Retrieve the most recent view.
        /// <p>
        /// Every time that SetView is called, the view is saved in this property.
        /// </summary>
        /// <returns>The view; returned as SdiView.</returns>
        public SdiView CurrentView
        {
            get { return m_viewCurrent; }
        }

        /// <summary>
        /// Returns the current view that is active in a container in the main form.
        /// <p>
        /// Uses the name that was sent to the SetView method.
        /// </summary>
        /// <param name="container">The container that contains the active view.</param>
        /// <returns>The view; returned as SdiView.</returns>
        public SdiView GetActiveView(Control container)
        {
            try
            {
                return hashActive[container];
            }
            catch (KeyNotFoundException)
            {
                return null;
            }
        }

        /// <summary>
        /// Property that returns the document, as SdiDoc.
        /// </summary>
        public SdiDoc Doc
        {
            get
            {
                return sdiDoc;
            }
            set
            {
                sdiDoc = value;
            }
        }

    }
}
