using System;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Generic;

namespace SdiTemplate
{
    /// <summary>
    /// Application context for Sdi application.<p>
    /// <b>Application context for Sdi application.</b><br>
    /// Call CreateDocument from main form of application.<br>
    /// Call SetView for each view to be added to main form.<br>
    /// The doc can be accessed through this context.<br>
    /// The active view can be accessed through this context.<br>
	/// </summary>
    
	public class SdiTemplate
	{
		protected SdiDoc sdiDoc;
        protected Dictionary<Control, SdiView> hashActive = 
            new Dictionary<Control, SdiView>();
        protected SdiMainForm m_mainForm;

        /// <summary>
        /// Default Constructor.
        /// <p>
        /// The default constructor does not do anything, but it is safer to
        /// always create a default constructor: the designer calls them at
        /// the strangest times.
        /// </summary>
		public SdiTemplate()
		{
            m_mainForm = null;
		}

         /// <summary>
        /// Constructor that sets the main form.
        /// <p>
        /// Call this constructor from the main form to create the SDI template
        /// for this main form. 
        /// </summary>
        public SdiTemplate(SdiMainForm mainForm)
        {
            this.m_mainForm = mainForm;
        }

        /// <summary>
        /// Calls constructor of type, using reflection.
        /// <p>
        /// If a view has not been created, then it can be created in the
        /// base class using reflection. This saves the programmer from 
        /// having to construct each view before adding it to a container.
        /// </summary>
        /// <param name="name">The name to be used to retrieve this view.</param>
        /// <param name="type">The type of the class that for this view.</param>
        /// <returns>Returns the view that is created as SdiView.</returns>
        protected SdiView CreateView(String name, Type type)
        {
            Object[] args = new Object[] { name, this };
            SdiView viewToCreate = (SdiView)Activator.CreateInstance(type, args);
            return viewToCreate;
        }

        #region Public members to be called from main form

        /// <summary>
        /// Call this method from the main form of an Sdi application to create the 
        /// document for the application.
        /// </summary>
        /// <param name="type">The type of the document class.</param>
        /// <returns>The document that was created, as SdiDoc.</returns>
		public SdiDoc CreateDocument(Type type)
		{
			Object[] args = new Object[]{this};
			Doc = (SdiDoc) Activator.CreateInstance (type, args);
			return Doc;
		}

        /// <summary>
        /// Make a view active in a container.
        /// <p>
        /// This will construct the view if it has not already been constructed.<br>
        /// Call it for each view that will be displayed.<br>
        /// This method will make this view active and hide the previous view.
        /// </summary>
        /// <param name="strViewName">The name used to retrieve the view.</param>
        /// <param name="type">The type of the view class.</param>
        /// <param name="control">The container control that the view should be added to.</param>
        /// <returns>The view that was activated; returned as SdiView.</returns>
		public SdiView SetView(String strViewName, Type type, Control control) 
		{
			SdiView viewToSet = GetView(strViewName, type, control);
			if (viewToSet == null) 
			{
				viewToSet = CreateView(strViewName, type);
			}
			if (!control.Controls.Contains(viewToSet)) 
			{
				control.Controls.Add(viewToSet);
			}
			SdiView active = GetActiveView(control);
			if (active != null) active.Hide();
			hashActive[control] = viewToSet;
			//viewToSet.Update(null);
			viewToSet.BringToFront();
			viewToSet.Show();
			return viewToSet;
        }

        /// <summary>
        /// Retrieve a view by name.
        /// <p>
        /// Uses the name that was sent to the SetView method.
        /// </summary>
        /// <param name="key">The name of the view to retrieve.</param>
        /// <param name="type">The type of the view to retrieve.</param>
        /// <param name="container">The container that contains the view.</param>
        /// <returns>The view; returned as SdiView.</returns>
        public SdiView GetView(String key, Type type, Control container)
        {
            foreach (Control control in container.Controls)
            {
                SdiView view = control as SdiView;
                if (view != null && view.Name == key)
                {
                    return view;
                }
            }
            return null;
        }

        /// <summary>
        /// Returns the current view that is active in a container in the main form.
        /// <p>
        /// Uses the name that was sent to the SetView method.
        /// </summary>
        /// <param name="container">The container that contains the active view.</param>
        /// <returns>The view; returned as SdiView.</returns>
		public SdiView GetActiveView(Control container) 
		{
            try
            {
                return hashActive[container];
            }
            catch (KeyNotFoundException)
            {
                return null;
            } 
	    }

        /// <summary>
        /// Property that returns the document, as SdiDoc.
        /// </summary>
		public SdiDoc Doc
		{
			get
			{
				return sdiDoc;
			}
			set
			{
				sdiDoc = (SdiDoc) value; 
			}
        }

        /// <summary>
        /// Property that returns the accesses the main form.
        /// </summary>
        public SdiMainForm MainForm
        {
            get { return m_mainForm; }
            set { m_mainForm = value; }
        }

        #endregion

    }
}
