//Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

//This source code is intended only as a supplement to Microsoft
//Development Tools and/or on-line documentation.  See these other
//materials for detailed information regarding Microsoft code samples.

//THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//PARTICULAR PURPOSE.
using System.Diagnostics;

namespace SdiTemplate
{
	using System;
	using System.IO ;
	using System.Drawing;
	using System.Drawing.Printing;
	using System.Collections;
	using System.ComponentModel;
	using System.Windows.Forms;

	/// <summary>
	/// SdiView is a user control. 
    /// All views should extend from this class. 
	/// </summary>
	public class SdiView : UserControl
	{
        private SdiMainForm m_parent;

        /// <summary>
        ///    Default Constructor.
        /// <p>
        /// The default constructor is required for the visual studio
        /// designer. The default constructor should not be called
        /// when a new view is created in an application.
        /// <p>
        /// Even though the SdiView is a user control, do not add it to
        /// the toolbox. The designer will call the default contructor.
        /// It is necessary to call the constructor that has parameters,
        /// instead.
        /// </summary>
        public SdiView()
        {
            SdiInitializeComponent();
        }

        //
        //
        /// <summary>
        /// Create a constructor in all views with this signature.
        /// <p>
        /// To create a class for the SdiView, create an empty class and
        /// extend it from SdiView. Call the SetView method to add the
        /// view to a container control.
        /// <p>
        /// Use this constructor to create your user control. Do not use
        /// the designer to add the SdiView to your project. The designer
        /// will call the default constructor, which will cause errors in
        /// your application.
        /// <p>
        /// Be sure to call the base class constructor with the same arguments.
        /// <pre>
        /// public ExtendedView(String name, SdiMainForm parent)
        ///     : base(name, parent)
        /// {
        /// 	...
        /// }
        /// </pre>
        /// If you modify the extended view in the designer, be sure to
        /// call InitializeComponent in the extended view from the view's constructor.
        /// InitializeComponent will not be created until you modify the extended
        /// view in the designer.
        /// <pre>
        /// public ExtendedView(String name, SdiApplicationContext context)
        ///     : base(name, parent)
        /// {
        /// 	...
        ///     InitializeComponent();
        ///     ...
        /// }
        /// </pre>
        /// </summary>
        /// <param name="name">The name to be used to retrieve the view</param>
        /// <param name="context">The main form that will show this view</param>
        public SdiView(String name, SdiMainForm parent)
            : this()
        {
            Name = name;
            this.m_parent = parent;
            m_parent.Doc.Update += new UpdateEvent(Update);
            this.Paint += new PaintEventHandler(SdiView_Paint);
            Hide();
        }

        protected override void Dispose(bool disposing)
        {
            m_parent.Doc.Update -= Update;
            Paint -= SdiView_Paint;
            base.Dispose(disposing);
        }


        #region Methods to override in the extended class

        /// <summary>
        ///    Override in derived class for actions to perform when doc changes.
        /// <p>
        /// Be sure to call the base class Doc_Update or call Invalidate(true) in
        /// the overriden method.
        /// </summary>
        /// <param name="sender">The view that initiated the change to the doc.</param>
        public virtual void Update(Object sender)
        {
            Invalidate(true);
        }

        /// <summary>
        ///    Override this method in the extended class.
        /// <p>
        /// Each view that is created with SdiApplication.SetView is subscribed
        /// to the paint event. Override this method in the extended class to
        /// handled painting. It is not necessary to subscribe to the paint handler
        /// in the extended class.
        /// </summary>
        /// <param name="sender">Object that initiated paint.</param>
        /// <param name="e">Normal event args for paint</param>
        public virtual void SdiView_Paint(object sender, PaintEventArgs e)
        {
            //override in extended class for polymorphic painting
        }

        #endregion

        /// <summary>
        ///    Default appearance of view.
        /// <p>
        /// This is the default appearance of view before any changes are made
        /// in the extended class. This method should not be changed or
        /// overridden. Make all changes in extended view class. Any changes
        /// made to the extended class in the designer will be reflected in IntializeComponent in the 
        /// extended class.
        /// </summary>
        protected void SdiInitializeComponent()
        {
            this.SuspendLayout();
             
            this.BackColor = System.Drawing.SystemColors.ActiveCaption;
            this.Name = "SdiView";
            this.Size = new System.Drawing.Size(266, 287);
            this.ResumeLayout(false);

        }

        public new SdiMainForm Parent
        {
            get { return m_parent; }
        }
	}
}
